# Generated by Django 3.2.19 on 2023-05-25 13:39

from django.db import migrations, models
import django_walletpass.storage


class Migration(migrations.Migration):

    dependencies = [
        ('django_walletpass', '0007_auto_20190613_1807'),
    ]

    operations = [
        migrations.AlterField(
            model_name='pass',
            name='data',
            field=models.FileField(storage=django_walletpass.storage.WalletPassStorage(), upload_to='uploads/passes/%Y/%m/%d/'),
        ),
    ]
